import React, { useEffect, useState, useRef } from "react";
import { useRecordWebcam } from "react-record-webcam";
import { Bars } from "react-loader-spinner";

import {
  webcamOffIcon,
  webcamOnIcon,
  micOffIcon,
  micOnIcon,
} from "../../utils/SvgIcons";
import { ButtonGeneric } from "../_reusables";
import Timer from "./Timer";
import "./_CamMicTest.scss";

const CamMicTest = ({ setTest }) => {
  const [isCamSet, setCam] = useState(null);
  const [isCamTest, setCamTest] = useState(false);
  const [isMicSet, setMic] = useState(null);
  const [isMicTest, setMicTest] = useState(false);
  const [micError, setMicError] = useState("");
  const [camError, setCamError] = useState("");
  const [isNext, setIsNext] = useState(false);
  const [sampleVid, setSampleVid] = useState(null);
  const [time, setTime] = useState(10);
  const [isRecording, setRecording] = useState(false);
  const [videoStream, setVideoStream] = useState();

  const videoRef = useRef();

  const recordWebcam = useRecordWebcam({
    recordingLength: time,
    fileType: "mp4",
    width: 500,
    height: 400,
  });

  const handleNext = () => {
    if (!isNext) {
      setIsNext(true);
      recordWebcam.open();
      return;
    }

    if (isNext) {
      recordWebcam.close();
      setTest(false);

      if (videoStream) {
        videoStream.stop();
      }
    }
  };

  const handleRecord = () => {
    setRecording(true);
    recordWebcam.start();
  };

  const handlePlay = () => {
    videoRef.current.play();
  };

  const handleRetest = () => {
    setTime(10);
    setSampleVid(null);
    recordWebcam.open();
  };

  const handleSampleVid = async () => {
    const blob = await recordWebcam.getRecording();

    const file = new File([blob], `test-record.mp4`, {
      type: "video/mp4",
    });

    const reader = new FileReader();

    reader.readAsDataURL(file);

    reader.onload = (e) => {
      const videoFile = e.target.result;
      setSampleVid(videoFile);
      setRecording(false);
    };

    recordWebcam.close();
  };

  useEffect(() => {
    if (!isNext) {
      navigator.mediaDevices
        .getUserMedia({ audio: true })
        .then((stream) => {
          const mic = stream.getAudioTracks();
          if (mic[0].enabled) {
            setMic(true);
            setMicTest(true);
          }
        })
        .catch((err) => {
          setMic(false);
          setMicTest(true);
          if (
            err.name === "NotAllowedError" ||
            err.name == "PermissionDeniedError"
          ) {
            setMicError(
              "Make sure that the granted access permission is enabled."
            );
            return;
          }

          if (
            err.name === "NotReadableError" ||
            err.name == "TrackStartError"
          ) {
            setMicError("Microphone is already in use in other application.");
            return;
          }

          if (
            err.name === "NotFoundError" ||
            err.name == "DevicesNotFoundError"
          ) {
            setMicError("No microphone connected on your machine.");
            return;
          }
        });

      navigator.mediaDevices
        .getUserMedia({ video: true })
        .then((stream) => {
          const vid = stream.getVideoTracks();
          if (vid[0].enabled) {
            setCam(true);
            setCamTest(true);
            setVideoStream(stream);
          }
        })
        .catch((err) => {
          setCam(false);
          setCamTest(true);
          if (
            err.name === "NotAllowedError" ||
            err.name == "PermissionDeniedError"
          ) {
            setCamError(
              "Make sure that the granted access permission is enabled."
            );
            return;
          }

          if (
            err.name === "NotReadableError" ||
            err.name == "TrackStartError"
          ) {
            setCamError("Camera is already in use in other application.");
            return;
          }

          if (
            err.name === "NotFoundError" ||
            err.name == "DevicesNotFoundError"
          ) {
            setCamError("No camera connected on your machine.");
            return;
          }
        });
    }

    if (time === 0) {
      handleSampleVid();
    }
  }, [time, sampleVid]);

  return (
    <div className="cam-mic-test-container">
      <div className="cam-mic-test-sub-container">
        {!isNext ? (
          <div className="cam-mic-container">
            {isMicTest && isCamTest ? (
              <>
                <div className="test-container">
                  <div>
                    {isCamSet
                      ? webcamOnIcon("#7964b5")
                      : webcamOffIcon("#aeadb4")}
                  </div>
                  <p className={`${!isCamSet ? "hasError" : ""}`}>Web camera</p>
                  {!isCamSet && <p className="error-test">{camError}</p>}
                  {isCamSet && (
                    <p className="success-test">
                      No issue with your current device.
                    </p>
                  )}
                </div>
                <div className="test-container">
                  <div>
                    {isMicSet ? micOnIcon("#7964b5") : micOffIcon("#aeadb4")}
                  </div>
                  <p className={`${!isMicSet ? "hasError" : ""}`}>Microphone</p>
                  {!isMicSet && <p className="error-test">{micError}</p>}
                  {isMicSet && (
                    <p className="success-test">
                      No issue with your current device.
                    </p>
                  )}
                </div>
              </>
            ) : (
              <div className="cam-mic-loader">
                <div>
                  <Bars height="40" width="80" color="#7964b5" />
                </div>
                <p>Checking camera and microphone...</p>
              </div>
            )}
          </div>
        ) : (
          <div className="sample-test-container">
            <div className="sample-rec-container">
              {sampleVid && (
                <video ref={videoRef} src={sampleVid} height={400} />
              )}
              {!sampleVid && (
                <video
                  ref={recordWebcam.webcamRef}
                  autoPlay
                  muted
                  height={400}
                />
              )}
            </div>
            {!isRecording && time === 10 && (
              <>
                <p>Record a 10 seconds clip to see how you look and sound.</p>
                <ButtonGeneric
                  text="Record a short clip"
                  size="small"
                  event={handleRecord}
                />
              </>
            )}

            {!isRecording && time === 0 && (
              <div className="play-retest-btn-container">
                <ButtonGeneric text="Play" size="small" event={handlePlay} />
                <ButtonGeneric
                  text="Re Test"
                  hasOutline={true}
                  size="small"
                  event={handleRetest}
                />
              </div>
            )}

            {isRecording && (
              <>
                <p>
                  Say something, no one else will see this and it will not be
                  saved
                </p>
                <div className="timer-test-container">
                  <Timer initLength={time} setInitLength={setTime} />
                </div>
              </>
            )}
          </div>
        )}
        <div className="test-btn-container">
          <ButtonGeneric
            text={isNext ? "Done" : "Next"}
            size="extraSmall"
            hasOutline={true}
            event={handleNext}
          />
        </div>
      </div>
    </div>
  );
};

export default CamMicTest;
