import React, { useContext, useState, useRef, useEffect } from "react";
import AudioReactRecorder, { RecordState } from "audio-react-recorder";
import Swal from "sweetalert2";
import { useParams, useHistory } from "react-router-dom";
import { Oval } from "react-loader-spinner";

import { AssesmentContext } from "../../context";
import { MicrophoneIcon, StopIcon } from "../../utils/SvgIcons";
import { getUserData, replaceValueSpaceHyphen } from "../../utils/Common";
import { ButtonGeneric, CustomRadio } from "../../components/_reusables";
import { saveTextAssesment, getTextSpeechCategoryScore } from "../../api";
import "./_AssesmentExam.scss";

const AssesmentExam = () => {
  const {
    currentAssesment,
    setCurrentAssesment,
    categories,
    setCategories,
    setCompletedCount,
    userTextOutput,
    setRender,
  } = useContext(AssesmentContext);
  const [currentSlide, setCurrentSlide] = useState(0);
  const [currentText, setCurrentText] = useState(null);
  const [isRecording, setIsRecording] = useState(null);
  const [isSaving, setSaving] = useState(false);
  const [currRecordData, setCurrRecordData] = useState(null);
  const [currScore, setCurrScore] = useState(null);

  const [choice, setChoice] = useState(null);
  const [hasChoicesDone, setHasChoicesDone] = useState(false);
  const audioRef = useRef();

  const fromLocal = getUserData();
  const { textId } = useParams();
  const history = useHistory();

  const handleRecordStart = () => setIsRecording(RecordState.START);
  const handleRecordStop = () => setIsRecording(RecordState.STOP);
  const handlePlay = () => audioRef.current.play();

  const handleOnStop = (audioData) => {
    // setSaving(true);
    setCurrRecordData(null);
    setCurrScore(null);
    const file = new File([audioData.blob], "record.mp3", {
      type: "audio/mp3",
    });
    const reader = new FileReader();

    reader.readAsDataURL(file);

    reader.onload = (e) => {
      const audioFile = e.target.result;
      setCurrRecordData({
        url: audioFile,
        type: audioData.type,
        data: audioData,
      });
    };
  };

  const handleSaveRecord = () => {
    setSaving(true);
    saveTextAssesment(
      {
        user_id: fromLocal.id,
        text: currentAssesment?.text[currentSlide]?.text,
        text_id: currentAssesment?.text[currentSlide]?.id,
        recording: currRecordData.url,
      },
      fromLocal.token
    )
      .then((res) => {
        if (res.data.success) {
          setSaving(false);
          setCurrentSlide(currentSlide + 1);
          setCurrScore(res.data.data);
        }
      })
      .catch((err) => {
        setSaving(false);
        if (
          err.response.status === 401 ||
          err.response.status === 422 ||
          err.response.status === 500 ||
          err.response.status === 404 ||
          err.response.status === 405
        ) {
          Swal.fire({
            title: "Error!",
            text: err.response.data.message,
            icon: "error",
            confirmButtonText: "Close",
            confirmButtonColor: "#55428A",
          });
          return;
        }

        if (err.response.status === 500) {
          console.log(err.response.data.message);
        }
      });
  };

  const handleNext = () => {
    setCurrRecordData(null);
    setCurrScore(null);
    setChoice(null);
    history.push(
      `/${replaceValueSpaceHyphen(fromLocal.first_name)}/language-assessment/${
        currentAssesment?.category
      }/${currentAssesment?.text[currentSlide]?.id}`
    );
  };

  const handleDone = async () => {
    setSaving(true);
    const res = await getTextSpeechCategoryScore(
      { user_id: fromLocal?.id, category_id: currentAssesment?.id },
      fromLocal?.token
    );

    if (
      res.status === 401 ||
      res.status === 402 ||
      res.status === 500 ||
      res.status === 404 ||
      res.status === 405
    ) {
      setSaving(false);
      Swal.fire({
        title: "Error!",
        text: res.data.message,
        icon: "error",
        confirmButtonText: "Close",
        confirmButtonColor: "#55428A",
      });
      return;
    }

    // if (res.status === 500) {
    //   setSaving(false);
    //   console.log(res.status);
    //   return;
    // }

    if (res.data.success) {
      setSaving(false);
      const {
        quality_score_average,
        fluency_score_average,
        pte_score_average,
        ielts_score_average,
      } = res.data.data;

      const currentCat = categories.filter(
        (item) => item.id === currentAssesment.id
      );
      const currentCatIndex = categories.findIndex(
        (item) => item.id === currentAssesment.id
      );

      const categoriesInstance = categories;

      categoriesInstance.splice(currentCatIndex, 1, {
        ...currentCat[0],
        score: {
          quality_score_average,
          fluency_score_average,
          pte_score_average,
          ielts_score_average,
        },
      });

      Swal.fire({
        title: `${
          currentCat[0].category === "Pronounciation"
            ? "Pronunciation"
            : currentCat[0].category
        } completed.`,
        icon: "success",
        customClass: {
          htmlContainer: "swal-average-assesment-score",
        },
        html: `<p>Your total score in ${
          currentCat[0].category === "Pronounciation"
            ? "pronunciation"
            : currentCat[0].category.toLowerCase()
        } category:</p>
        <div>
          <div>
            <div>
              <span>${Math.round(parseInt(fluency_score_average))}</span>
            </div>
            <p>Fluency</p>
          </div>
          <div>
            <div>
              <span>${Math.round(parseInt(quality_score_average))}</span>
            </div>
            <p>Quality</p>
          </div>
          <div>
            <div>
              <span>${Math.round(parseInt(ielts_score_average))}</span>
            </div>
            <p>PTE</p>
          </div>
          <div>
            <div>
              <span>${Math.round(parseInt(ielts_score_average))}</span>
            </div>
            <p>IELTS</p>
          </div>
        </div>
        `,
        confirmButtonText: "Ok",
        confirmButtonColor: "#55428A",
      }).then(() => {
        setCategories(categoriesInstance);
        setCompletedCount((prev) => prev + 1);
        setCurrentAssesment(null);
        setCurrRecordData(null);
        setCurrScore(null);
        history.push(
          `/${replaceValueSpaceHyphen(
            fromLocal?.first_name
          )}/language-assessment`
        );
      });
    }
  };

  const handleBack = () => {
    setRender((prev) => !prev);
    history.push(
      `/${replaceValueSpaceHyphen(fromLocal?.first_name)}/language-assessment`
    );
  };

  useEffect(() => {
    if (currentAssesment) {
      const text = currentAssesment?.text?.filter(
        ({ id }) => id === parseInt(textId)
      );

      const userTextLength = userTextOutput.current?.filter(
        ({ category }) => category === currentAssesment?.category
      );

      const newCurrentSlide =
        userTextOutput.current.length > 0 &&
        userTextLength[0]?.text?.length > 0 &&
        userTextLength[0]?.text?.length > currentSlide
          ? userTextLength[0]?.text?.length
          : currentSlide;
      setCurrentText(text[0]);
      setHasChoicesDone(text[0].choices.length === 0 ? false : true);
      setCurrentSlide(newCurrentSlide);
    }
  }, [textId]);

  return (
    <div>
      <div className="assesment-exam-count">
        <span>{`${currentSlide} of ${currentAssesment?.text?.length} ${
          currentAssesment?.category === "Pronounciation"
            ? "pronunciation"
            : currentAssesment?.category.toLowerCase()
        } completed`}</span>
        <div style={{ width: 200 }}>
          <ButtonGeneric
            text="Back to categories"
            size="small"
            hasOutline={true}
            event={handleBack}
          />
        </div>
      </div>
      <div className="assesment-exam-list">
        {choice ? (
          <p>
            <span>
              {currentText?.text.slice(0, currentText?.text.indexOf("_"))}
            </span>
            <span className="selected-choice">{choice}</span>
            <span>
              {currentText?.text.slice(
                currentText?.text.lastIndexOf("_") + 1,
                currentText?.text.length
              )}
            </span>
          </p>
        ) : (
          <p>{currentText?.text}</p>
        )}
        {hasChoicesDone ? (
          <>
            <p className="assesment-choice-header">
              Choose the word or phrase that best completes the sentence from
              the given alternatives.
            </p>
            <div className="assesment-choices-main-container">
              <div className="assesment-choices-container">
                {/* {currentText?.choices
                ?.filter((item) => item.choice !== choice)
                .map((item, i) => (
                  <span key={i} onClick={() => setChoice(item.choice)}>
                    {item.choice}
                  </span>
                ))} */}
                {currentText?.choices.map((item, i) => (
                  <CustomRadio
                    key={item.choice}
                    name="choice"
                    event={(e) => setChoice(item.choice)}
                    text={item.choice}
                  />
                ))}
              </div>
              {choice && (
                <div className="assesment-exam-btn-container">
                  <ButtonGeneric
                    text="Continue"
                    size="small"
                    hasOutline={true}
                    event={() => setHasChoicesDone(false)}
                  />
                </div>
              )}
            </div>
          </>
        ) : (
          <>
            {!currRecordData && (
              <p className="assesment-exam-btn-instruct">
                Please click the mic button and read the sentence clearly to
                record.
              </p>
            )}
            <div className="assesment-exam-btn-container">
              <button
                className="assesment-record-btn"
                onClick={
                  currScore
                    ? null
                    : isRecording === "start"
                    ? handleRecordStop
                    : handleRecordStart
                }
              >
                {isRecording === "start" ? (
                  StopIcon("#f53a29")
                ) : isSaving ? (
                  <div
                    style={{
                      display: "flex",
                      alignItems: "center",
                      justifyContent: "center",
                    }}
                  >
                    <Oval
                      height={27}
                      width={27}
                      strokeWidth={5}
                      color="#55428a"
                      secondaryColor="#f0eef7"
                    />
                  </div>
                ) : (
                  MicrophoneIcon("#55428a")
                )}
              </button>
            </div>
            <div className="assesment-exam-sub-btn-container">
              {currRecordData && isRecording === "stop" && (
                <ButtonGeneric
                  text="Play audio recording"
                  size="small"
                  hasOutline={true}
                  event={handlePlay}
                />
              )}
              {currRecordData && isRecording === "stop" && (
                <ButtonGeneric
                  text={
                    currScore === null
                      ? "Save record / Show result"
                      : currentSlide === currentAssesment?.text?.length &&
                        currScore !== null
                      ? "Done"
                      : "Continue"
                  }
                  color={"blue"}
                  size="small"
                  hasOutline={
                    currScore === null
                      ? true
                      : currentSlide === currentAssesment?.text?.length
                      ? false
                      : true
                  }
                  event={
                    currScore === null
                      ? handleSaveRecord
                      : currentSlide === currentAssesment?.text?.length
                      ? handleDone
                      : handleNext
                  }
                />
              )}
            </div>
            {currScore && isRecording === "stop" && (
              <>
                <div className="assesment-scores-container">
                  <div className="score-container">
                    <div>
                      <span>{currScore?.fluency_score}</span>
                      <span> /100</span>
                    </div>
                    <p>Fluency</p>
                  </div>
                  <div className="score-container">
                    <div>
                      <span>{currScore?.quality_score}</span>
                      <span> /100</span>
                    </div>
                    <p>Quality</p>
                  </div>
                  <div className="score-container">
                    <div>
                      <span>{currScore?.ielts_score}</span>
                      <span> /9</span>
                    </div>
                    <p>IELTS</p>
                  </div>
                  <div className="score-container">
                    <div>
                      <span>{currScore?.pte_score}</span>
                      <span> /90</span>
                    </div>
                    <p>PTE</p>
                  </div>
                </div>
              </>
            )}
            {currRecordData && (
              <audio
                ref={audioRef}
                style={{ display: "none" }}
                src={currRecordData.url}
                type={currRecordData.type}
                controls
              />
            )}
            <AudioReactRecorder
              state={isRecording}
              type="audio/mp3"
              backgroundColor="#fff"
              foregroundColor="#55428a"
              onStop={handleOnStop}
              canvasHeight={200}
            />
          </>
        )}
      </div>
    </div>
  );
};

export default AssesmentExam;
